@@ -10,7 +10,7 @@ import (
 	"github.com/restic/restic/internal/debug"
 	"github.com/restic/restic/internal/restic"
 
-	"bazil.org/fuse/fs"
+	"github.com/anacrolix/fuse/fs"
 )
 
 // Config holds settings for the fuse mount.
